@@ -9,7 +9,7 @@ from xml.dom import minidom
 
 import requests
 
-__version__ = "2.0.0.43"
+__version__ = "2.0.0.45"
 
 # internal module helper variables and functions
 _pma_sessions = dict()
@@ -43,6 +43,8 @@ def _pma_first_session_id():
 				# _pma_sessions[_pma_pmacoreliteSessionID] = _pma_pmacoreliteURL
 				
 				_pma_slideinfos[_pma_pmacoreliteSessionID] = dict()
+			if (not  _pma_pmacoreliteSessionID in _pma_amount_of_data_downloaded):
+				_pma_amount_of_data_downloaded[_pma_pmacoreliteSessionID] = 0
 			return _pma_pmacoreliteSessionID
 		else:
 			# no stored PMA.core sessions found NOR PMA.core.lite
@@ -305,7 +307,7 @@ def get_tile_size(sessionID = None):
 	if (len(_pma_slideinfos[sessionID]) < 1):
 		dir = get_first_non_empty_directory(sessionID)
 		slides = get_slides(dir, sessionID)
-		info = get_slide_info(sessionID, slides[0])
+		info = get_slide_info(slides[0], sessionID)
 	else:
 		info = choice(list(_pma_slideinfos[sessionID].values()))
 		
@@ -323,7 +325,6 @@ def get_slide_info(slideRef, sessionID = None):
 
 	if (not (slideRef in _pma_slideinfos[sessionID])):
 		url = _pma_api_url(sessionID, False) + "GetImageInfo?SessionID=" + _pma_q(sessionID) +  "&pathOrUid=" + _pma_q(slideRef)
-		print(url);
 		r = requests.get(url)
 		json = r.json()
 		global _pma_amount_of_data_downloaded 
@@ -435,7 +436,10 @@ def get_number_of_layers(slideRef, sessionID = None):
 	info = get_slide_info(slideRef, sessionID)
 	layers = info["TimeFrames"][0]["Layers"]
 	return len(layers)
-	
+
+def get_number_of_z_stack_layers(slideRef, sessionID = None):
+ 	return get_number_of_layers(slideRef, sessionID)
+
 def is_fluorescent(slideRef, sessionID = None):
 	"""Determine whether a slide is a fluorescent image or not"""
 	return get_number_of_channels(slideRef, sessionID) > 1
@@ -469,6 +473,7 @@ def get_barcode_url(slideRef, sessionID = None):
 
 def get_barcode_image(slideRef, sessionID = None):
 	"""Get the barcode (alias for "label") image for a slide"""
+	sessionID = _pma_session_id(sessionID)
 	r = requests.get(get_barcode_url(slideRef, sessionID))
 	img = Image.open(BytesIO(r.content))
 	global _pma_amount_of_data_downloaded 
@@ -481,6 +486,7 @@ def get_label_url(slideRef, sessionID = None):
 	
 def get_label_image(slideRef, sessionID = None):
 	"""Get the label image for a slide"""
+	sessionID = _pma_session_id(sessionID)
 	r = requests.get(get_label_url(slideRef, sessionID))
 	img = Image.open(BytesIO(r.content))
 	global _pma_amount_of_data_downloaded 
@@ -497,7 +503,9 @@ def get_thumbnail_url(slideRef, sessionID = None):
 	
 def get_thumbnail_image(slideRef, sessionID = None):
 	"""Get the thumbnail image for a slide"""
-	r = requests.get(get_thumbnail_url(slideRef, sessionID))
+	sessionID = _pma_session_id(sessionID)
+	url = get_thumbnail_url(slideRef, sessionID)
+	r = requests.get(url)
 	img = Image.open(BytesIO(r.content))
 	global _pma_amount_of_data_downloaded 
 	_pma_amount_of_data_downloaded[sessionID] += len(r.content)
@@ -521,14 +529,15 @@ def get_tile(slideRef, x = 0, y = 0, zoomlevel = None, zstack = 0, sessionID = N
 		+ "?SessionID=" + _pma_q(sessionID)
 		+ "&channels=" + _pma_q("0")
 		+ "&timeframe=" + _pma_q("0")
-		+ "&layer=" + _pma_q(zstack)
+		+ "&layer=" + str(int(round(zstack)))
 		+ "&pathOrUid=" + _pma_q(slideRef)
-		+ "&x=" + _pma_q(x)
-		+ "&y=" + _pma_q(y)
-		+ "&z=" + _pma_q(zoomlevel)	
+		+ "&x=" + str(int(round(x)))
+		+ "&y=" + str(int(round(y)))
+		+ "&z=" + str(int(round(zoomlevel)))
 		+ "&format=" + _pma_q(format)
 		+ "&quality=" + _pma_q(quality)
 		+ "&cache=" + str(_pma_usecachewhenretrievingtiles).lower())
+
 	r = requests.get(url)
 	img = Image.open(BytesIO(r.content))
 	global _pma_amount_of_data_downloaded 
@@ -571,5 +580,4 @@ def show_slide(slideRef, sessionID = None):
 			url += ("viewer/index.htm"
 			+ "?sessionID=" + _pma_q(sessionID)
 			+ "^&pathOrUid=" + _pma_q(slideRef))    # note the ^& to escape a regular &
-	os.system(os_cmd+url)
-6
\ No newline at end of file
+	os.system(os_cmd+url)
\ No newline at end of file
